@@ -2,6 +2,7 @@
 # (c) 2019-2021 Andreas Motl <andreas@hiveeyes.org>
 # License: GNU Affero General Public License, Version 3
 import asyncio
+import dataclasses
 import json
 import logging
 from collections import OrderedDict
@@ -11,13 +12,14 @@ from urllib.parse import urljoin, urlparse
 import colored
 import requests
 import requests_cache
-from munch import Munch, munchify
+from munch import Munch, munchify, unmunchify
 from tqdm import tqdm
 
 from grafana_wtf.model import (
     DashboardDetails,
     DashboardExplorationItem,
     DatasourceExplorationItem,
+    DatasourceItem,
     GrafanaDataModel,
 )
 from grafana_wtf.monkey import monkeypatch_grafana_api
@@ -193,6 +195,9 @@ class GrafanaEngine:
         if self.progressbar:
             self.taqadum.close()
 
+        # Improve determinism by returning stable sort order.
+        self.data.dashboards = munchify(sorted(self.data.dashboards, key=lambda x: x["dashboard"]["uid"]))
+
         return self.data.dashboards
 
     def handle_grafana_error(self, ex):
@@ -207,7 +212,7 @@ class GrafanaEngine:
     def fetch_dashboard(self, dashboard_info):
         log.debug(f'Fetching dashboard "{dashboard_info["title"]}" ({dashboard_info["uid"]})')
         dashboard = self.grafana.dashboard.get_dashboard(dashboard_info["uid"])
-        self.data.dashboards.append(munchify(dashboard))
+        self.data.dashboards.append(dashboard)
         if self.taqadum is not None:
             self.taqadum.update(1)
 
@@ -387,9 +392,9 @@ class GrafanaWtf(GrafanaEngine):
         # Compute list of exploration items, associating datasources with the dashboards that use them.
         results_used = []
         results_unused = []
-        for name in sorted(ix.datasource_by_name):
-            datasource = ix.datasource_by_name[name]
-            dashboard_uids = ix.datasource_dashboard_index.get(name, [])
+        for datasource in ix.datasources:
+            ds_identifier = datasource.get("uid", datasource.get("name"))
+            dashboard_uids = ix.datasource_dashboard_index.get(ds_identifier, [])
             dashboards = list(map(ix.dashboard_by_uid.get, dashboard_uids))
             item = DatasourceExplorationItem(datasource=datasource, used_in=dashboards, grafana_url=self.grafana_url)
 
@@ -399,7 +404,11 @@ class GrafanaWtf(GrafanaEngine):
             if dashboard_uids:
                 results_used.append(result)
             else:
-                results_unused.append(result)
+                if result not in results_unused:
+                    results_unused.append(result)
+
+        results_used = sorted(results_used, key=lambda x: x["datasource"]["name"] or x["datasource"]["uid"])
+        results_unused = sorted(results_unused, key=lambda x: x["datasource"]["name"] or x["datasource"]["uid"])
 
         response = OrderedDict(
             used=results_used,
@@ -419,18 +428,20 @@ class GrafanaWtf(GrafanaEngine):
         for uid in sorted(ix.dashboard_by_uid):
 
             dashboard = ix.dashboard_by_uid[uid]
-            datasource_names = ix.dashboard_datasource_index[uid]
+            datasource_items = ix.dashboard_datasource_index[uid]
 
             datasources_existing = []
-            datasource_names_missing = []
-            for datasource_name in datasource_names:
-                if datasource_name == "-- Grafana --":
+            datasources_missing = []
+            for datasource_item in datasource_items:
+                if datasource_item.name == "-- Grafana --":
                     continue
-                datasource = ix.datasource_by_name.get(datasource_name)
+                datasource_by_uid = ix.datasource_by_uid.get(datasource_item.uid)
+                datasource_by_name = ix.datasource_by_name.get(datasource_item.name)
+                datasource = datasource_by_uid or datasource_by_name
                 if datasource:
                     datasources_existing.append(datasource)
                 else:
-                    datasource_names_missing.append({"name": datasource_name})
+                    datasources_missing.append(dataclasses.asdict(datasource_item))
             item = DashboardExplorationItem(
                 dashboard=dashboard, datasources=datasources_existing, grafana_url=self.grafana_url
             )
@@ -439,8 +450,8 @@ class GrafanaWtf(GrafanaEngine):
             result = item.format_compact()
 
             # Add information about missing data sources.
-            if datasource_names_missing:
-                result["datasources_missing"] = datasource_names_missing
+            if datasources_missing:
+                result["datasources_missing"] = datasources_missing
 
             results.append(result)
 
@@ -453,6 +464,8 @@ class Indexer:
 
         # Prepare index data structures.
         self.dashboard_by_uid = {}
+        self.datasource_by_ident = {}
+        self.datasource_by_uid = {}
         self.datasource_by_name = {}
         self.dashboard_datasource_index = {}
         self.datasource_dashboard_index = {}
@@ -469,12 +482,16 @@ class Indexer:
         self.index_datasources()
 
     @staticmethod
-    def collect_datasource_names(element):
-        names = []
+    def collect_datasource_items(element):
+        items = []
         for node in element:
             if "datasource" in node and node["datasource"]:
-                names.append(node.datasource)
-        return list(sorted(set(names)))
+                ds = node.datasource
+                if isinstance(ds, Munch):
+                    ds = dict(ds)
+                if ds not in items:
+                    items.append(ds)
+        return sorted(items)
 
     def index_dashboards(self):
 
@@ -493,21 +510,38 @@ class Indexer:
             self.dashboard_by_uid[uid] = dashboard
 
             # Map to data source names.
-            ds_panels = self.collect_datasource_names(dbdetails.panels)
-            ds_annotations = self.collect_datasource_names(dbdetails.annotations)
-            ds_templating = self.collect_datasource_names(dbdetails.templating)
-            self.dashboard_datasource_index[uid] = list(sorted(set(ds_panels + ds_annotations + ds_templating)))
+            ds_panels = self.collect_datasource_items(dbdetails.panels)
+            ds_annotations = self.collect_datasource_items(dbdetails.annotations)
+            ds_templating = self.collect_datasource_items(dbdetails.templating)
+
+            results = []
+            for bucket in ds_panels, ds_annotations, ds_templating:
+                for item in bucket:
+                    item = DatasourceItem.from_payload(item)
+                    if item not in results:
+                        results.append(item)
+            self.dashboard_datasource_index[uid] = results
 
     def index_datasources(self):
 
+        self.datasource_by_ident = {}
+        self.datasource_by_uid = {}
         self.datasource_by_name = {}
         self.datasource_dashboard_index = {}
 
         for datasource in self.datasources:
-            name = datasource.name
-            self.datasource_by_name[name] = datasource
+            self.datasource_by_ident[datasource.name] = datasource
+            self.datasource_by_name[datasource.name] = datasource
+            if "uid" in datasource:
+                self.datasource_by_ident[datasource.uid] = datasource
+                self.datasource_by_uid[datasource.uid] = datasource
 
-        for dashboard_uid, datasource_names in self.dashboard_datasource_index.items():
-            for datasource_name in datasource_names:
-                self.datasource_dashboard_index.setdefault(datasource_name, [])
-                self.datasource_dashboard_index[datasource_name].append(dashboard_uid)
+        for dashboard_uid, datasource_items in self.dashboard_datasource_index.items():
+            datasource_item: DatasourceItem
+            for datasource_item in datasource_items:
+                datasource_name_or_uid = datasource_item.uid or datasource_item.name
+                if datasource_name_or_uid in self.datasource_by_name:
+                    if "uid" in self.datasource_by_name[datasource_name_or_uid]:
+                        datasource_name_or_uid = self.datasource_by_name[datasource_name_or_uid].uid
+                self.datasource_dashboard_index.setdefault(datasource_name_or_uid, [])
+                self.datasource_dashboard_index[datasource_name_or_uid].append(dashboard_uid)
